param ( [string]$lab, [string]$labno, [string]$detach )

if ($lab -eq "")
  {
    Write-host "Usage:" $MyInvocation.MyCommand "<lab> [number] [detach|keeprunning]"
    exit
  }

if ($labno -eq "detach")
  {
    $labno = $detach
    $detach = "detach"
  }
if ($labno -eq "keeprunning")
  {
    $labno = $detach
    $detach = "keeprunning"
  }

$file = "$lab\settings.txt"
$settings = "$lab\$labno\settings.txt"
if ((Test-Path -Path $file) -and (Test-Path -Path $settings))
  {
    Write-host "Using $settings, ignoring settings in ${file}"
  }
if (Test-Path -Path $settings)
  {
    $file = $settings
  }
if (-not (Test-Path -Path $file))
  {
    Write-host "Can't find lab $lab"
    exit
  }

# note: not used at this time:
$dockerargs = "--rm -t --pull never --sysctl net.ipv6.conf.all.disable_ipv6=0 --cap-add cap_net_admin --cap-add cap_net_raw --cap-add cap_sys_admin"
$path = Get-Location
$conrtr = ".\connectrouter.ps1"
$stoprtr = ".\connectrouter.ps1"
$dockerimage = "frrouting/frr:v8.3.1"
$configdir = "/etc/frr"
$startup = ""

$labno0 = $labno
foreach($line in Get-Content "$file")
{
  $splitup = -split $line
  if ($splitup[0] -eq $labno0 -or $splitup[0] -eq "default")
    {
      if ($splitup[1] -eq "dockerimage")
        {
          $dockerimage = $splitup[2..($splitup.length)]
        }
      if ($splitup[1] -eq "configdir")
        {
          $configdir = $splitup[2..($splitup.length)]
        }
      if ($splitup[1] -eq "startup")
        {
          $startup = $splitup[2..($splitup.length)]
        }
      if ($splitup[1] -eq "lab")
        {
          $lab = $splitup[2]
          $labno = $splitup[3]
        }
      if ($splitup[1] -eq "detach")
        {
          $attach = ""
        }
      if ($splitup[1] -eq "attach")
        {
          $attach = $splitup[2..($splitup.length)]
        }
      if ($splitup[1] -eq "routers")
        {
          $routers = $splitup[2..($splitup.length)]
        }
      if ($splitup[1] -eq "txt")
        {
          $txt = $splitup[2..($splitup.length)]
          $txt = $txt -replace "#conrtr#", ".\connectrouter.ps1"
          $txt = $txt -replace "#stoprtr#", ".\stoprouters.ps1"
          Write-host $txt
        }
    }
}

if ($routers -eq "")
  {
    Write-host "Example $example not found"
    exit
  }

$imgexists = docker image inspect ${dockerimage} --format="true"
if ($imgexists -ne "true")
  {
    Write-host
    $answer = Read-Host "Docker image ${dockerimage} is not installed. Do you want to pull (download) it? "
    if ($answer -eq "y" -or $answer -eq "Y" -or $answer -eq "yes" -or $answer -eq "Yes")
      {
        docker pull $dockerimage
      }
    else
      {
        exit
      }
  }

if ($labno -eq "")
  {
    $labpath="$path\${lab}\"
  }
else
  {
    $labpath="$path\$lab\$labno\"
    $labpath0="$path\$lab\0\"
  }

$running = docker ps --format "{{.Names}}"

Write-host -NoNewLine "Starting routers: "
foreach($router in -split "$routers")
  {
    $extrasetup = "yes"
    if (Test-Path -Path "$labpath$router\vlans.sh")
      {
       if ($running -like "*Router$router*")
          {
            docker stop -t 0 Router$router | Out-Null
          }
        Write-host -NoNewLine "$router "
        docker run --rm -t -l bgpminilab --pull never --sysctl net.ipv6.conf.all.disable_ipv6=0 --cap-add cap_net_admin --cap-add cap_net_raw --cap-add cap_sys_admin "-v" ${labpath}${router}:${configdir} --name Router${router} -d ${dockerimage} | Out-Null
        $stoprouters = "${stoprouters}${router} "
      }
    else
      {
        if ($running -like "*Router$router*")
          {
            Write-host -NoNewLine "{$router} "
            $extrasetup = ""
          }
        else
          {
            Write-host -NoNewLine "($router) "
            docker run --rm -t -l bgpminilab --pull never --sysctl net.ipv6.conf.all.disable_ipv6=0 --cap-add cap_net_admin --cap-add cap_net_raw --cap-add cap_sys_admin "-v" ${labpath0}${router}:${configdir} --name Router${router} -d ${dockerimage} | Out-Null
          }
        $keeprouters = "${keeprouters}${router} "
      }
    if ($extrasetup -ne "")
      {
        docker exec -d Router${router} bash ${configdir}/vlans.sh
        if ($startup -ne "")
          {
            $startup2 = ${startup}
            if (Test-Path -Path "$labpath$router\ospfd.conf")
              {
                $startup2 = "$startup2; ospfd -d"
              }
            if (Test-Path -Path "$labpath$router\ospf6d.conf")
              {
                $startup2 = "$startup2; ospf6d -d"
              }
            docker exec Router${router} sh -c "${startup2}"
          }
      }
  }
Write-host

if ($attach -ne "" -and $detach -ne "detach")
  {
    docker exec -it Router${attach} vtysh
    Write-host -NoNewLine "Stopping routers: "
    foreach($router in -split "$stoprouters")
      {
        Write-host -NoNewLine "$router "
        docker stop -t 0 Router$router | Out-Null
      }
  }

if ($attach -ne "" -and $detach -ne "detach" -and $detach -ne "keeprunning")
  {
    foreach($router in -split "$keeprouters")
      {
        Write-host -NoNewLine "$router "
        docker stop -t 0 Router$router | Out-Null
      }
  }
Write-host
if ($attach -ne "" -and $detach -eq "keeprunning")
  {
    Write-host "Note: routers ${keeprouters}are still running."
  }
