#!/bin/sh

if [ "$1" = "" ]; then
  echo "Usage: $0 <lab> [detach|keeprunning] [lab number]"
  exit 0
fi

lab="$1"
labno="$2"

if [ "$2" = "detach" ]; then
  labno="$3"
  detach="detach"
else
  if [ "$3" = "detach" ]; then
    labno="$2"
    detach="detach"
  fi
fi
if [ "$2" = "keeprunning" ]; then
  labno="$3"
  detach="keeprunning"
else
  if [ "$3" = "keeprunning" ]; then
    labno="$2"
    detach="keeprunning"
  fi
fi

file="${lab}/settings.txt"
settings="${lab}/${labno}/settings.txt"
if [ -e "$file" ] && [ -e "$settings" ] ; then
  echo "Using ${settings}, ignoring settings in ${file}"
fi
if [ -e "$settings" ] ; then
  file=$settings
fi
if [ ! -e "$file" ]; then
  echo "Can't find lab ${lab}"
  exit
fi

dockerargs="--rm -t --pull never --sysctl net.ipv6.conf.all.disable_ipv6=0 --cap-add cap_net_admin --cap-add cap_net_raw --cap-add cap_sys_admin"
# FRR
dockerimage="frrouting/frr:v8.3.1"
configdir="/etc/frr"
startup=""

labno0=${labno}
while read i
do
  num=`echo $i | cut -d' ' -f1`
  if [ \( "${num}" = "${labno0}" \) -o \( "${num}" = "default" \) ]; then
    action=`echo $i | cut -d' ' -f2`

    if [ "$action" = "dockerimage" ]; then
      dockerimage=`echo $i | cut -d' ' -f3-`
    fi
    if [ "$action" = "configdir" ]; then
      configdir=`echo $i | cut -d' ' -f3-`
    fi
    if [ "$action" = "startup" ]; then
      startup=`echo $i | cut -d' ' -f3-`
    fi

    if [ "$action" = "lab" ]; then
      lab=`echo $i | cut -d' ' -f3`
      labno=`echo $i | cut -d' ' -f4`
    fi
    if [ "$action" = "detach" ]; then
      attach=""
    fi
    if [ "$action" = "txt" ]; then
      echo $i | cut -d' ' -f3- | sed 's/#conrtr#/\.\/connectrouter.sh/' | sed 's/#stoprtr#/\.\/stoprouters.sh/'
    fi
    if [ "$action" = "attach" ]; then
      attach=`echo $i | cut -d' ' -f3`
    fi
    if [ "$action" = "routers" ]; then
      routers=`echo $i | cut -d' ' -f3-`
    fi
  fi
done < ${file}

imgexists=`docker image inspect ${dockerimage} --format="true" 2>/dev/null`
if [ "$imgexists" != "true" ]; then
  printf "\nDocker image ${dockerimage} is not installed. Do you want to pull (download) it? "
  read answer
  if [ "$answer" == "y" ] || [ "$answer" == "Y" ] || [ "$answer" == "yes" ] || [ "$answer" == "Yes" ]; then
    docker pull $dockerimage
  else
    exit
  fi
fi

if [ "$labno" = "" ]; then
  labpath="`pwd`/${lab}/"
else
  labpath="`pwd`/${lab}/${labno}/"
  labpath0="`pwd`/${lab}/0/"
fi

quagga=`docker ps -f ancestor=iljitsch/bgp2021 --format "{{.Names}}"`
frr=`docker ps -f ancestor=frrouting/frr:v8.3.1 --format "{{.Names}}"`
anyimage=`docker ps -f ancestor=$dockerimage --format "{{.Names}}"`
running="${quagga} ${frr} ${anyimage}"

if [ "${routers}" != "" ]; then
  printf "Starting routers: "
  for router in ${routers}
    do
      extrasetup="yes"
      if [ -e "$labpath$router/vlans.sh" ]; then
        if [[ $running == *"Router$router"* ]]; then
          docker stop -t 0 Router$router >/dev/null
        fi
        printf "$router "
        docker run $dockerargs -v "${labpath}${router}:${configdir}" --name Router${router} -d ${dockerimage} >/dev/null
        stoprouters="${stoprouters}${router} "
      else
        if [[ $running == *"Router$router"* ]]; then
          printf "{$router} "
          extrasetup=""
        else
          printf "($router) "
          docker run $dockerargs -v "${labpath0}${router}:${configdir}" --name Router${router} -d ${dockerimage} >/dev/null
        fi
      keeprouters="${keeprouters}${router} "
      fi
      if [ "$extrasetup" != "" ]; then
        docker exec -d Router${router} bash ${configdir}/vlans.sh
        if [ "${startup}" != "" ]; then
          startup2="${startup}"
          if [ -e "$labpath$router/ospfd.conf" ]; then
            startup2="${startup2}; ospfd -d"
          fi
          if [ -e "$labpath$router/ospf6d.conf" ]; then
            startup2="${startup2}; ospf6d -d"
          fi
          docker exec Router${router} sh -c "${startup2}"
        fi
      fi

      #defaultroute=`docker exec -it Router${router} ip route show default`
      #echo docker exec -it Router${router} ip route del ${defaultroute}
      #for vlanid in $vlans
      #  do
      #    printf "$vlanid "
      #    docker exec -it -d Router${router} ip link add link eth0 name eth0.$vlanid type vlan id $vlanid
      #  done
      #docker exec -it Router${router} ip route del `docker exec -it Router${router} ip route show default | tr -d '\r'`
    done
  echo

  if [ "${attach}" != "" ] && [ "$detach" != "detach" ]; then
    docker exec -it Router${attach} vtysh
    printf "Stopping routers: "
    for router in ${stoprouters}
      do
        printf "$router "
        docker stop -t 0 Router$router >/dev/null
      done
  fi
  if [ "${attach}" != "" ] && [ "$detach" != "detach" ] && [ "$detach" != "keeprunning" ]; then
  # docker ps -f ancestor=$dockerimage --format "{{.Names}}" |
  # while read x; do
  #   docker stop -t 0 $x >/dev/null
  # done
    for router in ${keeprouters}
      do
        printf "$router "
        docker stop -t 0 Router$router >/dev/null
      done
  fi
  echo
  if [ "${attach}" != "" ] && [ "$detach" = "keeprunning" ] && [ "$keeprouters" != "" ]; then
      echo "Note: routers ${keeprouters}are still running."
  fi
else
  echo "Couldn't find lab $lab $labno"
fi

